"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useContentEditableEffect = void 0;
var path_1 = require("@formily/path");
var events_1 = require("../events");
function placeCaretAtEnd(el, isCollapse) {
    var currentSelection = window.getSelection();
    if (currentSelection.containsNode(el))
        return;
    el.focus();
    var range = document.createRange();
    range.selectNodeContents(el);
    if (isCollapse) {
        range.collapse(false);
    }
    var sel = window.getSelection();
    sel.removeAllRanges();
    sel.addRange(range);
}
var useContentEditableEffect = function (engine) {
    var globalState = {
        activeElements: new Map(),
        requestTimer: null,
        isComposition: false,
    };
    function onKeyDownHandler(event) {
        if (event.key === 'Enter') {
            event.stopPropagation();
            event.preventDefault();
        }
    }
    function onInputHandler(event) {
        var _this = this;
        var node = globalState.activeElements.get(this);
        event.stopPropagation();
        event.preventDefault();
        if (node) {
            var target_1 = event.target;
            clearTimeout(globalState.requestTimer);
            globalState.requestTimer = setTimeout(function () {
                if (globalState.isComposition)
                    return;
                var isCollapsed = window.getSelection().isCollapsed;
                path_1.Path.setIn(node.props, _this.getAttribute(engine.props.contentEditableAttrName), target_1 === null || target_1 === void 0 ? void 0 : target_1.textContent);
                setTimeout(function () {
                    placeCaretAtEnd(_this, isCollapsed);
                }, 16);
            }, 1000);
        }
    }
    function onCompositionHandler(event) {
        if (event.type === 'compositionend') {
            globalState.isComposition = false;
            onInputHandler(event);
        }
        else {
            clearTimeout(globalState.requestTimer);
            globalState.isComposition = true;
        }
    }
    function onPastHandler(event) {
        event.preventDefault();
        var text = event.clipboardData.getData('text');
        this.textContent = text;
    }
    function findTargetNodeId(element) {
        if (!element)
            return;
        var nodeId = element.getAttribute(engine.props.contentEditableNodeIdAttrName);
        if (nodeId)
            return nodeId;
        var parent = element.closest("*[" + engine.props.nodeIdAttrName + "]");
        if (parent)
            return parent.getAttribute(engine.props.nodeIdAttrName);
    }
    engine.subscribeTo(events_1.MouseClickEvent, function (event) {
        var _a;
        var target = event.data.target;
        var editableElement = (_a = target === null || target === void 0 ? void 0 : target.closest) === null || _a === void 0 ? void 0 : _a.call(target, "*[" + engine.props.contentEditableAttrName + "]");
        if (editableElement &&
            editableElement.getAttribute('contenteditable') === 'true')
            return;
        globalState.activeElements.forEach(function (node, element) {
            globalState.activeElements.delete(element);
            element.setAttribute('contenteditable', 'false');
            element.removeEventListener('input', onInputHandler);
            element.removeEventListener('compositionstart', onCompositionHandler);
            element.removeEventListener('compositionupdate', onCompositionHandler);
            element.removeEventListener('compositionend', onCompositionHandler);
            element.removeEventListener('past', onPastHandler);
        });
    });
    engine.subscribeTo(events_1.MouseDoubleClickEvent, function (event) {
        var _a;
        var target = event.data.target;
        var editableElement = (_a = target === null || target === void 0 ? void 0 : target.closest) === null || _a === void 0 ? void 0 : _a.call(target, "*[" + engine.props.contentEditableAttrName + "]");
        var workspace = engine.workbench.activeWorkspace;
        var tree = workspace.operation.tree;
        if (editableElement) {
            var editable = editableElement.getAttribute('contenteditable');
            if (editable === 'false' || !editable) {
                var nodeId = findTargetNodeId(editableElement);
                if (nodeId) {
                    var targetNode = tree.findById(nodeId);
                    if (targetNode) {
                        globalState.activeElements.set(editableElement, targetNode);
                        editableElement.setAttribute('contenteditable', 'true');
                        editableElement.focus();
                        editableElement.addEventListener('input', onInputHandler);
                        editableElement.addEventListener('compositionstart', onCompositionHandler);
                        editableElement.addEventListener('compositionupdate', onCompositionHandler);
                        editableElement.addEventListener('compositionend', onCompositionHandler);
                        editableElement.addEventListener('keydown', onKeyDownHandler);
                        editableElement.addEventListener('paste', onPastHandler);
                        placeCaretAtEnd(editableElement, false);
                    }
                }
            }
        }
    });
};
exports.useContentEditableEffect = useContentEditableEffect;
